"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.getDefinitionForClass = exports.Service = void 0;
const serviceMetadata_1 = require("../serviceMetadata");
require("reflect-metadata");
const randomName_1 = require("../randomName");
const DEFINITION_KEY = Symbol('__alphaDic-ServiceDefinition');
exports.Service = function (name) {
    return function (constructor) {
        const finalName = name || randomName_1.randomName(constructor.name);
        const metadata = serviceMetadata_1.ensureMetadata(constructor);
        metadata.name = finalName;
        const definition = serviceMetadata_1.createDefinitionFromMetadata(metadata, constructor);
        Reflect.defineMetadata(DEFINITION_KEY, definition, constructor);
        if (exports.Service._container) {
            exports.Service._container.registerDefinition(definition);
        }
        else if (!process.env.ALPHA_DIC_NO_SERVICE_CONTAINER) {
            console.warn('There is no container registered in @Service decorator. ' +
                'Use Service.useContainer(container) to define the container. ' +
                'Without it service definitions created via @Service decorator cannot be registered automatically. ' +
                'If you wish to register service definitions manually via getDefinitionForClass set ALPHA_DIC_NO_SERVICE_CONTAINER environment variable to disable this warning.');
        }
    };
};
exports.Service.useContainer = function (container) {
    exports.Service._container = container;
};
function getDefinitionForClass(clazz) {
    return Reflect.getMetadata(DEFINITION_KEY, clazz);
}
exports.getDefinitionForClass = getDefinitionForClass;
